@@ -105,7 +105,6 @@ class ChromeRemoteDebugInterface(ChromeRemoteDebugInterface_base):
 
 
 		ret_2 = self.Network_setExtraHTTPHeaders(headers = header_args)
-
 		return (ret_1, ret_2)
 
 
@@ -113,6 +112,7 @@ class ChromeRemoteDebugInterface(ChromeRemoteDebugInterface_base):
 
 		ret = []
 
+		# This is kind of horrible
 		for item in js_object:
 			if 'name' in item:
 				if item['name'] == '__defineGetter__':
@@ -139,11 +139,172 @@ class ChromeRemoteDebugInterface(ChromeRemoteDebugInterface_base):
 					continue
 				if item['name'] == 'valueOf':
 					continue
+				if item['name'] == 'ABORT_ERR':
+					continue
+				if item['name'] == 'DATA_CLONE_ERR':
+					continue
+				if item['name'] == 'INUSE_ATTRIBUTE_ERR':
+					continue
+				if item['name'] == 'INVALID_ACCESS_ERR':
+					continue
+				if item['name'] == 'INVALID_CHARACTER_ERR':
+					continue
+				if item['name'] == 'INVALID_MODIFICATION_ERR':
+					continue
+				if item['name'] == 'INVALID_NODE_TYPE_ERR':
+					continue
+				if item['name'] == 'INVALID_STATE_ERR':
+					continue
+				if item['name'] == 'NAMESPACE_ERR':
+					continue
+				if item['name'] == 'NETWORK_ERR':
+					continue
+				if item['name'] == 'NO_DATA_ALLOWED_ERR':
+					continue
+				if item['name'] == 'NO_MODIFICATION_ALLOWED_ERR':
+					continue
+				if item['name'] == 'NOT_FOUND_ERR':
+					continue
+				if item['name'] == 'NOT_SUPPORTED_ERR':
+					continue
+				if item['name'] == 'QUOTA_EXCEEDED_ERR':
+					continue
+				if item['name'] == 'SECURITY_ERR':
+					continue
+				if item['name'] == 'SYNTAX_ERR':
+					continue
+				if item['name'] == 'TIMEOUT_ERR':
+					continue
+				if item['name'] == 'TYPE_MISMATCH_ERR':
+					continue
+				if item['name'] == 'URL_MISMATCH_ERR':
+					continue
+				if item['name'] == 'VALIDATION_ERR':
+					continue
+				if item['name'] == 'WRONG_DOCUMENT_ERR':
+					continue
+				if item['name'] ==  'DOMSTRING_SIZE_ERR':
+					continue
+				if item['name'] ==  'HIERARCHY_REQUEST_ERR':
+					continue
+				if item['name'] ==  'INDEX_SIZE_ERR':
+					continue
 
 			ret.append(item)
 
 		return ret
 
+	def __decode_serialized_value(self, value):
+		assert 'type' in value
+		assert 'value' in value
+
+		if value['type'] == 'number':
+			return float(value['value'])
+		if value['type'] == 'string':
+			return value['value']
+
+		# Special case for null/none objects
+		if (
+				    'subtype' in value
+				and
+				    value['subtype'] == 'null'
+				and
+				    value['type'] == 'object'
+				and
+				    value['value'] is None):
+			return None
+
+		self.log.warning("Unknown serialized javascript value of type %s", value['type'])
+		self.log.warning("Complete value: %s", value)
+
+		return value
+
+	def _unpack_xhr_resp(self, values):
+		ret = {}
+
+		for entry in values:
+			assert 'configurable' in entry, "'configurable' missing from entry (%s)" % entry
+			assert 'enumerable'   in entry, "'enumerable' missing from entry (%s)"   % entry
+			assert 'isOwn'        in entry, "'isOwn' missing from entry (%s)"        % entry
+			assert 'name'         in entry, "'name' missing from entry (%s)"         % entry
+			assert 'value'        in entry, "'value' missing from entry (%s)"        % entry
+			assert 'writable'     in entry, "'writable' missing from entry (%s)"     % entry
+
+			if entry['isOwn'] is False:
+				continue
+
+			assert entry['name'] not in ret
+			ret[entry['name']] = self.__decode_serialized_value(entry['value'])
+
+		return ret
+
+	def xhr_fetch(self, url, headers=None, post_data=None, post_type=None):
+		'''
+		Execute a XMLHttpRequest() for content at `url`. If
+		`headers` are specified, they must be a dict of string:string
+		keader:values. post_data must also be pre-encoded.
+
+		Note that this will be affected by the same-origin policy of the current
+		page, so it can fail if you are requesting content from another domain and
+		the current site has restrictive same-origin policies (which is very common).
+		'''
+
+		'''
+		If you're thinking this is kind of a hack, well, it is.
+
+		We also cheat a bunch and use synchronous XMLHttpRequest()s, because it
+		SO much easier.
+		'''
+		js_script = '''
+		function (url, headers, post_data, post_type){
+
+			var req = new XMLHttpRequest();
+
+			// We use sync calls, since we want to wait until the call completes
+			// This will probably be depreciated at some point.
+			if (post_data)
+			{
+				req.open("POST", url, false);
+				if (post_type)
+					req.setRequestHeader("Content-Type", post_type);
+			}
+			else
+				req.open("GET", url, false);
+
+			if (headers)
+			{
+				let entries = Object.entries(headers);
+				for (let idx = 0; idx < entries.length; idx += 1)
+				{
+					req.setRequestHeader(entries[idx][0], entries[idx][1]);
+
+				}
+			}
+
+			if (post_data)
+				req.send(post_data);
+			else
+				req.send();
+
+			return {
+					url          : url,
+					headers      : headers,
+					resp_headers : req.getAllResponseHeaders(),
+					post         : post_data,
+					response     : req.responseText,
+					mimetype     : req.getResponseHeader("Content-Type"),
+					code         : req.status
+				};
+		}
+
+		'''
+
+
+		ret = self.execute_javascript_function(js_script, [url, headers, post_data, post_type])
+		ret = self._unpack_xhr_resp(ret)
+		return ret
+		# if
+
 
 	def __unwrap_object_return(self, ret):
 		if "result" in ret and 'result' in ret['result']:
@@ -163,7 +324,7 @@ class ChromeRemoteDebugInterface(ChromeRemoteDebugInterface_base):
 		self.log.error("Failed fetching results from call!")
 		return ret
 
-	def __exec_js(self, script, should_call=False, args=None, **extra_params):
+	def __exec_js(self, script, should_call=False, args=None):
 		'''
 
 		Execute the passed javascript function/statement, optionally with passed
@@ -199,14 +360,14 @@ class ChromeRemoteDebugInterface(ChromeRemoteDebugInterface_base):
 		if args or should_call:
 			expression = "({script}).apply(null, JSON.parse({args}))".format(
 					script=script,
-					args=json.dumps(args)
+					args=repr(json.dumps(args))
 				)
 		else:
 			expression = "({script})".format(
 					script=script,
 				)
 
-		resp3 = self.Runtime_evaluate(expression=expression, **extra_params)
+		resp3 = self.Runtime_evaluate(expression=expression)
 
 		resp4 = self.__unwrap_object_return(resp3)
 
@@ -214,9 +375,6 @@ class ChromeRemoteDebugInterface(ChromeRemoteDebugInterface_base):
 
 
 
-
-
-
 	# Interact with http.cookiejar.Cookie() instances
 	def get_cookies(self):
 		'''
@@ -465,7 +623,8 @@ class ChromeRemoteDebugInterface(ChromeRemoteDebugInterface_base):
 
 		ret = self.__exec_js(script=script)
 		return ret
-	def execute_javascript_function(self, script, *args):
+
+	def execute_javascript_function(self, script, args=None):
 		'''
 		Execute a javascript function in the context of the browser tab.
 
@@ -473,7 +632,7 @@ class ChromeRemoteDebugInterface(ChromeRemoteDebugInterface_base):
 		be called via ({script}).apply(null, {args}).
 		'''
 
-		ret = self.__exec_js(script=script, should_call=True, *args)
+		ret = self.__exec_js(script=script, should_call=True, args=args)
 		return ret
 
 	def find_element(self, search):
@@ -792,7 +951,6 @@ class ChromeRemoteDebugInterface(ChromeRemoteDebugInterface_base):
 		Roughly, this corresponds to the javascript `DOMContentLoaded` event,
 		meaning the dom for the page is ready.
 
-
 		Internals:
 
 		A navigation command results in a sequence of events:
